var fs = require('fs');
var xml2js = require('xml2js');

var io;
var gameSocket;

var games = {};
var gameModes = {}

gameModes['reaction'] = require('./gameModes/reaction.js');
gameModes['completeness'] = require('./gameModes/completeness.js');

exports.initGameServer = function(sio, socket) {
    io = sio;
    gameSocket = socket;

    gameSocket.on('createGame', createGame);
    gameSocket.on('joinGame', joinGame);

    gameSocket.on('startGame', prepareQuestion);
    gameSocket.on('nextQuestion', prepareQuestion);
    gameSocket.on('submitAnswer', receiveAnswer);
}

function createGame(data) {
    var gameId = generateGameId();
    game = {
        id: gameId,
        state: "state",
        players: {},
        questions: [],
        currentQuestionIndex: -1,
        gameMode: data['gameMode']
    }

    games[gameId] = game;

    loadQuestions(game);

    this.join(gameId);

    broadcast(game, 'gameCreated', {gameId:gameId});
}

function joinGame(data) {
    var gameId = data['gameId'];
    var id = data['playerId'];
    var name = data['playerName'];

    if(games[gameId] != null) {
        games[gameId].players[id]= {
            id: id,
            name: name,
            answers: [],
            currentScore: 0
        };

        this.join(gameId);
    }

}

function prepareQuestion(data) {
    var game = games[data['gameId']];

    var questionPrepareTime = 3;

    broadcast(game, 'prepareQuestion', {
        time: questionPrepareTime,
    });

    setTimeout(function(){
        startQuestion(game);
    }, questionPrepareTime*1000 + 100);
}

function startQuestion(game) {

    game.currentQuestionIndex++;

    var question = game.questions[game.currentQuestionIndex];
    var d = new Date();
    question.startTime = d.getTime();

    broadcast(game, 'nextQuestion', {
        index: game.currentQuestionIndex,
        text: question.text,
        time: question.time,
        options: shuffle(question.decoys.concat(question.answers)),
        gameMode: game.gameMode
    });

    setTimeout(function() {
        questionEnded(game);
    }, question.time*1000 + 100);
}

function receiveAnswer(data) {
    var game = games[data['gameId']];
    var player = game.players[data['playerId']];
    var questionIndex = data['questionIndex'];

    if(questionIndex == game.currentQuestionIndex) {
        player.answers[questionIndex] = {
            options:data['answers'],
            time: timeSinceQuestionStart(game, questionIndex)
        }
    }
}

function questionEnded(game) {

    var scores = []

    for(var playerId in game.players) { 

        var player = game.players[playerId];

        var oldScore = player.currentScore;
        var newScore = gameModes[game.gameMode].calculateScore(game, player);

        player.currentScore = newScore;
        
        scores.push({
            playerId: player.id,
            playerName: player.name,
            score: newScore,
            scoreDifference: newScore - oldScore
        });
    };

    var index = game.currentQuestionIndex;
    var answers = game.questions[index].answers;




    scores.sort(function (a, b) { return b.score - a.score; });

    if(game.currentQuestionIndex == game.questions.length -1) {
        broadcast(game, "gameEnded", {scores: scores});
    }
    else {
        broadcast(game, "questionEnded", {
            scores: scores,
            answers: answers
        });
    }
}

function timeSinceQuestionStart(game, currentQuestionIndex) {
    var d = new Date();
    var currentTime = d.getTime();
    var questionStartTime = game.questions[currentQuestionIndex].startTime;

    return Math.round((currentTime - questionStartTime)/1000);
}

function generateGameId() {
    var length = 4;
    var id = '';
    var d = new Date();
    var time = d.getTime();
    
    for(var i = 0; i < length; i++) {
        id += (Math.floor(time+Math.random()*10) % 10);
        time = Math.floor(time/8);
    }   
    return id;
}

function shuffle(array) {
    var currentIndex = array.length;
    var temporaryValue;
    var randomIndex;

    // While there remain elements to shuffle...
    while (0 !== currentIndex) {

        // Pick a remaining element...
        randomIndex = Math.floor(Math.random() * currentIndex);
        currentIndex -= 1;

        // And swap it with the current element.
        temporaryValue = array[currentIndex];
        array[currentIndex] = array[randomIndex];
        array[randomIndex] = temporaryValue;
    }

    return array;
}

function broadcast(game, event, data) {
    io.sockets.in(game.id).emit(event, data);
}

function loadQuestions(game) {
    var questionFilePath = gameModes[game.gameMode].getQuestionFilePath();

    loadXML(questionFilePath, function (result) {
        for (var i = 0; i < result.questionPool.question.length; i++) {
            var question = result.questionPool.question[i];

            game.questions.push({
                text: question.text,
                time: parseInt(question.time),
                answers: question.answer,
                decoys: question.decoy
            });
        }
    });
}

function loadXML(filePath, callback) {
    try {
        var fileData = fs.readFileSync(filePath);
        var parser = new xml2js.Parser();

        parser.parseString(fileData, function (err, result) {
            callback(result);
        });

        console.log("File '" + filePath + "/ was successfully read.\n");
    } catch (ex) {
        console.log("Unable to read file '" + filePath + "'.");
        console.log(ex);
    }
}
